$ifndef _SPI_Modes_
$define _SPI_Modes_
'
' These macros contain implementations of Modes 0, 1, 2, and 3 Master SPI.
'
Declare Reminders = Off

    Dim PP0H As Byte System                     ' Use a system variable for the loop counter
    Dim PP0 As Byte System                      ' Use a system variable for the transfer variable

$ifndef SPI0_SCK
    $define SPI0_SCK PORTB.0                    ' \
    $define SPI0_MOSI PORTB.1                   ' | Set the IO lines for SPI Mode 0
    $define SPI0_MISO PORTB.2                   ' /
$endif

$ifndef SPI1_SCK
    $define SPI1_SCK PORTB.0                    ' \
    $define SPI1_MOSI PORTB.1                   ' | Set the IO lines for SPI Mode 1
    $define SPI1_MISO PORTB.2                   ' /
$endif

$ifndef SPI2_SCK
    $define SPI2_SCK PORTB.0                    ' \
    $define SPI2_MOSI PORTB.1                   ' | Set the IO lines for SPI Mode 2
    $define SPI2_MISO PORTB.2                   ' /
$endif

$ifndef SPI3_SCK
    $define SPI3_SCK PORTB.0                    ' \
    $define SPI3_MOSI PORTB.1                   ' | Set the IO lines for SPI Mode 3
    $define SPI3_MISO PORTB.2                   ' /
$endif

'-----------------------------------------------------------------------------
    GoTo Over_SPI_Macro_Subroutines             ' Jump over the subroutines
    
'-----------------------------------------------------------------------------
' Simultaneously transmits and receives one byte using the mode 0SPI protocol.
' SCK is idle-low, and bits are latched on SCK rising.
'
$define SPI_Out0(pBytein) _SPI_Out0 pBytein

_SPI_Out0 Macro pBytein
    #if(Prm_Count >= 1)
        #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
            Num_Wreg pBytein
        #else
            Byte_Wreg pBytein
        #endif
    #else
        Movlw 0
    #endif
    GoSub _SPI_Mode_0
    #if(_SPI_Out0_Return == 1)
        #if(Return_Type == Byte) || (Return_Type == SByte)
            Wreg_Byte Return_Var
        #endif
        #if(Return_Type == Word) || (Return_Type == SWord)
            Wreg_Word Return_Var
        #endif
        #if(Return_Type == Dword) || (Return_Type == SDword)
            Wreg_Dword Return_Var
        #endif
    #endif
    Endm

#ifdef _SPI_Out0#Req
'
' Input     : WREG = 8-Bit value to send
'           : SPI0_SCK holds the Port.Bit of the SPI Clock pin
'           : SPI0_MOSI holds the Port.Bit of the SPI Data In Pin
'           : SPI0_MISO holds the Port.Bit of the SPI Data Out Pin
' Output    : WREG = 8-Bit value received
'
_SPI_Mode_0:
    Output SPI0_SCK
    Output SPI0_MOSI
    Input SPI0_MISO

    Wreg_Byte PP0                           ' Transfer the WREG into PP0
    PP0H = 8                                ' Single byte SPI loop
_SPI_Mode0_LP:
        SPI0_MOSI = PP0 & $80               ' Put current Outgoing Bit on MOSI
        PP0 = PP0 << 1                      ' Shift next Bit into MSB
        SPI0_SCK = 1                        ' Set SCK high
        PP0 = PP0 | SPI0_MISO               ' Capture current Bit on MISO
        SPI0_SCK = 0                        ' Set SCK low
    Djnz PP0H,_SPI_Mode0_LP
    Movf PP0,w                              ' Transfer PP0 into WREG
    Return
#endif
    
'-----------------------------------------------------------------------------
' Simultaneously transmits and receives one byte using the mode 1 SPI protocol.
' SCK is idle-low, and bits are latched on SCK falling.
'
$define SPI_Out1(pBytein) _SPI_Out1 pBytein

_SPI_Out1 Macro pBytein
        #if(Prm_Count >= 1)
            #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
                Num_Wreg pBytein
            #else
                Byte_Wreg pBytein
            #endif
        #else
            Movlw 0
        #endif
        GoSub _SPI_Mode_1
        #if(_SPI_Out1_Return == 1)
            #if(Return_Type == Byte) || (Return_Type == SByte)
                Wreg_Byte Return_Var
            #endif
            #if(Return_Type == Word) || (Return_Type == SWord)
                Wreg_Word Return_Var
            #endif
            #if(Return_Type == Dword) || (Return_Type == SDword)
                Wreg_Dword Return_Var
            #endif
        #endif
        Endm

#ifdef _SPI_Out1#Req
'
' Input     : WREG = 8-Bit value to send
'           : SPI1_SCK holds the Port.Bit of the SPI Clock pin
'           : SPI1_MOSI holds the Port.Bit of the SPI Data In Pin
'           : SPI1_MISO holds the Port.Bit of the SPI Data Out Pin
' Output    : WREG = 8-Bit value received
'
_SPI_Mode_1:
        Output SPI1_SCK
        Output SPI1_MOSI
        Input SPI1_MISO

        Wreg_Byte PP0                           ' Transfer the WREG into PP0
        PP0H = 8                                ' Single byte SPI loop
_SPI_Mode1_LP:
            SPI1_SCK = 1                        ' Set SCK high
            SPI1_MOSI = PP0 & $80               ' Put current Outgoing Bit on MOSI
            PP0 = PP0 << 1                      ' Shift next Bit into MSB
            SPI1_SCK = 0                        ' Set SCK low
            PP0 = PP0 | SPI1_MISO               ' Capture current Bit on MISO
        Djnz PP0H,_SPI_Mode1_LP
        Movf PP0,w                              ' Transfer PP0 into WREG
        Return
#endif
    
'-----------------------------------------------------------------------------
' Simultaneously transmits and receives one byte using the mode 2 SPI protocol.
' SCK is idle-high, and bits are latched on SCK falling.
'
$define SPI_Out2(pBytein) _SPI_Out2 pBytein

_SPI_Out2 Macro pBytein
        #if(Prm_Count >= 1)
            #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
                Num_Wreg pBytein
            #else
                Byte_Wreg pBytein
            #endif
        #else
            Movlw 0
        #endif
        GoSub _SPI_Mode_2
        #if(_SPI_Out2_Return == 1)
            #if(Return_Type == Byte) || (Return_Type == SByte)
                Wreg_Byte Return_Var
            #endif
            #if(Return_Type == Word) || (Return_Type == SWord)
                Wreg_Word Return_Var
            #endif
            #if(Return_Type == Dword) || (Return_Type == SDword)
                Wreg_Dword Return_Var
            #endif
        #endif
        Endm

#ifdef _SPI_Out2#Req
'
' Input     : WREG = 8-Bit value to send
'           : SPI2_SCK holds the Port.Bit of the SPI Clock pin
'           : SPI2_MOSI holds the Port.Bit of the SPI Data In Pin
'           : SPI2_MISO holds the Port.Bit of the SPI Data Out Pin
' Output    : WREG = 8-Bit value received
'
_SPI_Mode_2:
        Output SPI2_SCK
        Output SPI2_MOSI
        Input SPI2_MISO

        Wreg_Byte PP0                           ' Transfer the WREG into PP0
        PP0H = 8                                ' Single byte SPI loop
_SPI_Mode2_LP:
            SPI2_MOSI = PP0 & $80               ' Put current Outgoing Bit on MOSI
            PP0 = PP0 << 1                      ' Shift next Bit into MSB
            SPI2_SCK = 0                        ' Set SCK high
            PP0 = PP0 | SPI2_MISO               ' Capture current Bit on MISO
            SPI2_SCK = 1                        ' Set SCK low
        Djnz PP0H,_SPI_Mode2_LP
        Movf PP0,w                              ' Transfer PP0 into WREG
        Return
#endif
    
'-----------------------------------------------------------------------------
' Simultaneously transmits and receives one byte using the mode 3 SPI protocol.
' SCK is idle-high, and bits are latched on SCK rising.
'
$define SPI_Out3(pBytein) _SPI_Out3 pBytein

_SPI_Out3 Macro pBytein
        #if(Prm_Count >= 1)
            #if((Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32))
                Num_Wreg pBytein
            #else
                Byte_Wreg pBytein
            #endif
        #else
            Movlw 0
        #endif
        GoSub _SPI_Mode_3
        #if(_SPI_Out3_Return == 1)
            #if(Return_Type == Byte) || (Return_Type == SByte)
                Wreg_Byte Return_Var
            #endif
            #if(Return_Type == Word) || (Return_Type == SWord)
                Wreg_Word Return_Var
            #endif
            #if(Return_Type == Dword) || (Return_Type == SDword)
                Wreg_Dword Return_Var
            #endif
        #endif
        Endm

#ifdef _SPI_Out3#Req
'
' Input     : WREG = 8-Bit value to send
'           : SPI3_SCK holds the Port.Bit of the SPI Clock pin
'           : SPI3_MOSI holds the Port.Bit of the SPI Data In Pin
'           : SPI3_MISO holds the Port.Bit of the SPI Data Out Pin
' Output    : WREG = 8-Bit value received
'
_SPI_Mode_3:
        Output SPI3_SCK
        Output SPI3_MOSI
        Input SPI3_MISO

        Wreg_Byte PP0                           ' Transfer the WREG into PP0
        PP0H = 8                                ' Single byte SPI loop
_SPI_Mode3_LP:
            SPI3_SCK = 0                        ' Set SCK low
            SPI3_MOSI = PP0 & $80               ' Put current Outgoing Bit on MOSI
            PP0 = PP0 << 1                      ' Shift next Bit into MSB
            SPI3_SCK = 1                        ' Set SCK high
            PP0  = PP0 | SPI3_MISO              ' Capture current Bit on MISO
        Djnz PP0H,_SPI_Mode3_LP
        Movf PP0,w                              ' Transfer PP0 into WREG
        Return
#endif
'-----------------------------------------------------------------------------

Over_SPI_Macro_Subroutines:
Declare Reminders = On
$endif

